/* 
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation

  STARTER PROGRAM

 * Section BH
 *
 */


#include "gp142.h"
#include <stdio.h>
#include <math.h>

#include <time.h>
	/* 
	* <time.h> is included so the current time can be accessed
	* to initialize the random number generator.  If you don't
	* use random numbers (and you don't have to) you can either
	* ignore this #include or delete it.
	*/

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0


/*
 * Symbolic constants.  Change as needed.
 */

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75

/* Number of buttons */
#define NUM_BUTTONS 	6


/* Possible locations of mouse clicks: */

/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0


/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER ((-GP142_YMAX + HORIZON) / 3)
#define HIGHWAY_WIDTH  10


/* Maximum number of vehicles that can appear in the animation */
#define MAX_VEHICLES	10

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40


/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y);

/* Draw the buttons */
void draw_buttons();

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *car_color, int *car_y);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int *x, int *y, int *direction, int speed);

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x, int y, int color);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color);

/* Display error message if internal error is detected. */
void something_is_wrong (void);

/* Initialize the variables for a new car. */
void initialize_new_car (int*car_x, int*car_y,
						int*car_directionon, int*car_color,		
						int*car_speed);

/*****************************************************************************

     MAIN

*****************************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */

	/* Information about vehicles in the animation:										*/
	/* [Only one car now - will need to be changed to add more]							*/
	int car_x[MAX_VEHICLES];			/* current x position of a car					*/
	int car_y[MAX_VEHICLES];			/* current y position of a car					*/
	int car_direction[MAX_VEHICLES];	/* current direction: +1=right, -1=left			*/
	int car_color[MAX_VEHICLES];		/* color of the car								*/
	int car_speed[MAX_VEHICLES];		/* speed in pixels per tick when traffic moving	*/	
	int traffic_moving;					/* = "traffic is currently moving"				*/
	int car_count = 0;					/* a counter for use with some loops.			*/
	int car_number = 1;					/* the number of cars currently involved.		*/
	int max_cars = FALSE;				/* is the number of cars at its max?			*/
	int car_id = 1;						/* a number id of the currently selected car.	*/

	initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;

	/* set initial conditions for the cars */
	
	initialize_new_car(&car_x[0], &car_y[0], &car_direction[0], 
	 				    &car_color[0], &car_speed[0]);

	traffic_moving = TRUE;

	
	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x,sun_y);
	draw_buttons( );
	draw_traffic(car_x[0], car_y[0], car_color[0]);
	

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* When a key is pressed, it'll add a new car if the number */
			/* of cars is less than the maximum, as set by MAX_VEHICLES */
			/* If the number is at max, it will simply scroll through   */
			/* the car id's, so that each car can be selected seperately*/
			/* to have the info (color, speed, etc.) changed.			*/
			
			car_id++;
			car_number++;
			
			if (car_number > MAX_VEHICLES){
				
				car_number--;
				max_cars = TRUE;
			}
				
			if (car_id > MAX_VEHICLES){
					car_id = 1;
			}
			
			if (max_cars == FALSE){
					initialize_new_car(&car_x[car_id-1], &car_y[car_id-1], 
	    						&car_direction[car_id-1], 
       	    					&car_color[car_id-1], 
								&car_speed[car_id-1]);
			}
		

			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, &traffic_moving, 
							   &car_speed[car_id-1], 
							   &car_direction[car_id-1], 
							   &car_color[car_id-1],
							   &car_y[car_id-1]);
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			move_sun(&sun_x, &sun_y);
			
			/* Loop to allow the movement of all cars involved. */
			while (car_count < car_number){
				if (traffic_moving)
					move_traffic(&car_x[car_count], &car_y[car_count], 
						&car_direction[car_count], car_speed[car_count]);
				
				car_count++;
				
			}
			car_count = 0;

			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y);
		draw_buttons();
		
		/* Looped to redraw all cars heading left */
		while (car_count < car_number){
			if(car_y[car_count] > HIGHWAY_CENTER){
				draw_traffic(car_x[car_count], car_y[car_count], 
					car_color[car_count]);
			/* Headlights! */
				GP142_rectangleXY(YELLOW, car_x[car_count] - CAR_WIDTH/2, 
						car_y[car_count] + CAR_HEIGHT - 7 ,
						car_x[car_count] - CAR_WIDTH/2 - 1, 
						car_y[car_count] + CAR_HEIGHT - 10 , FILL);
			
			}
						
							
			car_count++;
			
		}

		car_count = 0;
		/* So the cars on the bottom aren't covered by those on */
		/* top, this loop will draw the rest of the cars, those */
		/* heading right.										*/
		while (car_count < car_number){
			if(car_y[car_count] < HIGHWAY_CENTER){
				draw_traffic(car_x[car_count], car_y[car_count],
					car_color[car_count]);
				/* More Headlights! */
				GP142_rectangleXY(YELLOW, car_x[car_count] + CAR_WIDTH/2 + 1, 
						car_y[car_count] + CAR_HEIGHT - 7 ,
						car_x[car_count] + CAR_WIDTH/2 , 
						car_y[car_count] + CAR_HEIGHT - 10 , FILL);
			
			
			}
			car_count++;
		}
		
		car_count = 0;


	
		
	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return (0);
}  /*end main */


/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/


/* Draw the background, including the moon at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;			/* size of text font */
	int center_line_x = -GP142_XMAX;


	/* Draw the sky */
	GP142_rectangleXY(BLACK, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* This'll draw some stars to put into the night sky. */

	GP142_rectangleXY(WHITE, -GP142_XMAX +10, GP142_YMAX - 10,
		-GP142_XMAX + 12, GP142_YMAX - 8, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +100, GP142_YMAX - 50,
		-GP142_XMAX + 102, GP142_YMAX - 52, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +200, GP142_YMAX - 100,
		-GP142_XMAX + 202, GP142_YMAX - 102, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +150, GP142_YMAX - 30,
		-GP142_XMAX + 152, GP142_YMAX - 32, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +300, GP142_YMAX - 10,
		-GP142_XMAX + 302, GP142_YMAX - 12, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +450, GP142_YMAX - 140,
		-GP142_XMAX + 452, GP142_YMAX - 142, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +390, GP142_YMAX - 65,
		-GP142_XMAX + 392, GP142_YMAX - 67, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +465, GP142_YMAX - 50,
		-GP142_XMAX + 467, GP142_YMAX - 52, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +435, GP142_YMAX - 95,
		-GP142_XMAX + 437, GP142_YMAX - 97, FILL);
		GP142_rectangleXY(WHITE, -GP142_XMAX +500, GP142_YMAX - 85,
		-GP142_XMAX + 502, GP142_YMAX - 87, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +550, GP142_YMAX - 25,
		-GP142_XMAX + 552, GP142_YMAX - 27, FILL);
	GP142_rectangleXY(WHITE, -GP142_XMAX +240, GP142_YMAX - 40,
		-GP142_XMAX + 242, GP142_YMAX - 42, FILL);



	/* Draw the sun */
	GP142_circleXY(WHITE, sun_x, sun_y , 20);
	
	/* Draw the ground, highway, and text legend */
	GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	GP142_rectangleXY(22, 
					-GP142_XMAX, HIGHWAY_CENTER - HIGHWAY_WIDTH,
					 GP142_XMAX, HIGHWAY_CENTER + HIGHWAY_WIDTH,
					 FILL);
	
	//Here's the function to draw the line down the center
	//of the highway.

	while(center_line_x < GP142_XMAX){

		GP142_lineXY(YELLOW, center_line_x, HIGHWAY_CENTER, 
				 center_line_x + 15, HIGHWAY_CENTER, 2);
		center_line_x = center_line_x + 30;
	}

	
	GP142_printfXY (WHITE, text_offset, -GP142_YMAX+30, text_size,
		"Click a button to control the car.");
	
	/* This is just a printf for informative purposes. */
	GP142_printfXY(WHITE, text_offset, -GP142_YMAX+50, text_size + 1,
		"Press a key to add / select new car. ( Max %d )",
		  MAX_VEHICLES);
	 
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	
} /*end draw_landscape */



/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/


/* Draws the buttons */
void draw_buttons( )
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	int font_size = 12;				/* size of button text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(WHITE,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(3, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "U-Turn");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "New Color");
	
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *car_color, int *car_y)
{
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		
		//now it'll start the car up again if the "go" button
		//is pressed.
		
		*traffic_moving = TRUE;
			
		break;
	case 1:
		/* stop */
		*traffic_moving = FALSE;
		break;
		
	case 2:
		/* speed up */
		*car_speed = *car_speed + 1;
		break;
	case 3:
		/* slow down */
		*car_speed = *car_speed - 1;
		if (*car_speed < 0)
			*car_speed = 0;
		break;
		
	case 4:
		/* reverse */
		*car_direction = - *car_direction;
		if (*car_direction < 0){
				*car_y = *car_y + 15;
		}
		else {
			*car_y = *car_y - 15;
		}


		break;
	case 5:
		/* change car color */
		*car_color = (*car_color+1) % MAX_COLORS;
		break;
	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}



/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/


/* Update moon coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the moon to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	(*sun_x) --;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
	}
}


/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(int *x, int *y, int *direction, int speed) {
	/* Note: If there is more than one vehicle, this should */
	/* be updated to adjust all vehicle coordinates */
	
	/* Actually, I just had main run this function once */
	/* for each car on the screen, so I didn't need to  */
	/* fiddle with this function at all.				*/

	int trial_x;		/* new value for x coordinate */

	/* move vehicle.  If it moves off the edge, change its side*/
	trial_x = *x + (*direction * speed);
	
	if(abs(trial_x) > GP142_XMAX + 15){
		*x = -*x;
	}
	else
		*x = trial_x;
}


/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x, int y, int color) {
	
	draw_car(x,y,color);
}



/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color) {
	GP142_rectangleXY(color, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(BLACK, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, FILL);
	GP142_rectangleXY(color, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, 1);
	GP142_rectangleXY(color, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x , y + CAR_HEIGHT, 1);
	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
}



/*****************************************************************************

     OTHER

*****************************************************************************/


/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}



void initialize_new_car(int*car_x, int*car_y,
						int*car_direction, int*car_color,		
						int*car_speed){

	

	*car_x = GP142_XMAX + 15;
	*car_y = HIGHWAY_CENTER - 8;
	*car_direction = +1;
	*car_color = RED;
	*car_speed = 5;



}

		

